/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package chat.protocol;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.PushException;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.MessageManager;
import org.objectweb.dream.protocol.BindException;
import org.objectweb.dream.protocol.ExportException;
import org.objectweb.dream.protocol.IncomingPush;
import org.objectweb.dream.protocol.OutgoingPush;
import org.objectweb.dream.protocol.channel.ChannelFactory;
import org.objectweb.dream.protocol.channel.ChannelProtocol;
import org.objectweb.dream.protocol.channel.TCPIPProtocol;
import org.objectweb.dream.util.Error;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * 
 */
public class ChatServerImpl extends AbstractComponent
    implements
      ChannelFactory,
      IncomingPush
{

  Set<OutgoingPush> clients = new HashSet<OutgoingPush>();

  /** Default constructor. */
  public ChatServerImpl()
  {
    fcNeedAsyncStart = true;
  }

  // ---------------------------------------------------------------------------
  // Client interfaces
  // ---------------------------------------------------------------------------

  MessageManager  messageManagerItf;
  ChannelProtocol lowerProtocolItf;

  // ---------------------------------------------------------------------------
  // Implementation of the IncomingPush interface
  // ---------------------------------------------------------------------------

  /**
   * @see IncomingPush#incomingPush(Message)
   */
  public synchronized void incomingPush(Message message) throws PushException
  {
    System.out.println("has received a message to broadcast !!");
    // broadcast message
    Iterator<OutgoingPush> iterator = clients.iterator();
    while (iterator.hasNext())
    {
      OutgoingPush outgoingPush = iterator.next();
      try
      {
        outgoingPush.outgoingPush(message);
      }
      catch (PushException e)
      {
        // ignore
      }
    }
  }

  /**
   * @see IncomingPush#incomingClosed(Object, Exception)
   */
  public synchronized void incomingClosed(Object outgoingPush,
      Exception exception)
  {
    System.out.println("connection closed");
    if (!clients.remove(outgoingPush))
    {
      Error.bug(logger);
    }

  }

  // ---------------------------------------------------------------------------
  // Implementation of the Channel interface
  // ---------------------------------------------------------------------------

  /**
   * @see ChannelFactory#instantiate(OutgoingPush)
   */
  public synchronized IncomingPush instantiate(OutgoingPush toClientPush)
      throws BindException
  {
    System.out.println("has a new client !!!");
    clients.add(toClientPush);
    return this;
  }

  // ---------------------------------------------------------------------------
  // Implementation of BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{ChannelProtocol.LOWER_PROTOCOL_ITF_NAME,
        MessageManager.ITF_NAME};
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(ChannelProtocol.LOWER_PROTOCOL_ITF_NAME))
    {
      lowerProtocolItf = (ChannelProtocol) serverItf;
    }
    else if (clientItfName.equals(MessageManager.ITF_NAME))
    {
      messageManagerItf = (MessageManager) serverItf;
    }
  }

  // ---------------------------------------------------------------------------
  // Overridden methods
  // ---------------------------------------------------------------------------

  /**
   * @see AbstractComponent#startFc()
   */
  public void startFc() throws IllegalLifeCycleException
  {
    super.startFc();
    Map<String, Object> hints = new HashMap<String, Object>();
    hints.put(TCPIPProtocol.PORT, new Integer(3000));
    try
    {
      lowerProtocolItf.export((ChannelFactory) weaveableC
          .getFcInterface(CHANEL_ITF_NAME), hints);
    }
    catch (ExportException e)
    {
      throw new IllegalLifeCycleException(
          "Unable to export channel interface : " + e.getMessage());
    }
    catch (NoSuchInterfaceException e)
    {
      Error.bug(logger, e);
    }
  }
}
