/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.adl.type;

import org.objectweb.dream.adl.AbstractDelagatingChecker;
import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.Node;
import org.objectweb.fractal.adl.interfaces.Interface;
import org.objectweb.fractal.adl.interfaces.InterfaceContainer;
import org.objectweb.fractal.adl.types.TypeInterface;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.api.type.InterfaceType;

/**
 * Type checker for fractal component.
 */
public class FractalTypeChecker extends AbstractDelagatingChecker
{

  /**
   * @see AbstractDelagatingChecker#doChek(Object, Object)
   */
  protected void doChek(Object legacy, Object node) throws ADLException
  {
    ComponentType legacyType = (ComponentType) ((Component) legacy).getFcType();
    if (node instanceof InterfaceContainer)
    {
      Interface[] itfs = ((InterfaceContainer) node).getInterfaces();
      for (int i = 0; i < itfs.length; i++)
      {
        TypeInterface itfType = (TypeInterface) itfs[i];
        InterfaceType legacyItfType = getLegacyInterfaceType(legacyType,
            itfType);

        if (legacyItfType.isFcClientItf() != "client".equals(itfType.getRole()))
        {
          throw new ADLException(
              "Legacy interface and node interface has not the same role",
              (Node) itfType);
        }
        if (legacyItfType.isFcCollectionItf() != "collection".equals(itfType
            .getCardinality()))
        {
          throw new ADLException(
              "Legacy interface and node interface has not the same cardinality",
              (Node) itfType);
        }
      }
    }
  }

  InterfaceType getLegacyInterfaceType(ComponentType legacyType,
      TypeInterface adlType) throws ADLException
  {
    boolean collection = "collection".equals(adlType.getCardinality());
    InterfaceType[] interfaceTypes = legacyType.getFcInterfaceTypes();
    if (collection)
    {
      for (int i = 0; i < interfaceTypes.length; i++)
      {
        InterfaceType itfType = interfaceTypes[i];
        if (adlType.getName().startsWith(itfType.getFcItfName()))
        {
          return itfType;
        }
      }
    }
    else
    {
      for (int i = 0; i < interfaceTypes.length; i++)
      {
        InterfaceType itfType = interfaceTypes[i];
        if (adlType.getName().equals(itfType.getFcItfName()))
        {
          return itfType;
        }
      }
    }
    throw new ADLException(
        "The legacy component has not the specified interface", (Node) adlType);
  }
}