/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema, Romain Lenglet
 */

package org.objectweb.dream.aggregator;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.Pull;
import org.objectweb.dream.PullException;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.MessageManager;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * This aggregator has a collection of pull inputs, and a pull output. When a
 * message is pulled on the output, the aggregator pulls messages on its input,
 * aggregates them, and returns the aggregated message. Messages are pulled
 * sequentially in a non predictable order. Input pull interfaces can return
 * <code>null</code>.
 */
public class PullPullAggregatorCollectionInputImpl extends AbstractComponent
    implements
      Pull
{

  private Map<String, Pull> inPullItfs = new HashMap<String, Pull>();

  /** the message manager client interface of this component */
  protected MessageManager  messageManagerItf;

  /**
   * @see Pull#pull()
   */
  public Message pull() throws PullException
  {
    Message outMessage = messageManagerItf.createMessage();
    Iterator iter = inPullItfs.values().iterator();
    while (iter.hasNext())
    {
      Pull pullItf = (Pull) iter.next();
      Message msg = pullItf.pull();
      if (msg != null)
      {
        messageManagerItf.addSubMessage(outMessage, msg);
      }
    }
    return outMessage;
  }

  // ---------------------------------------------------------------------------
  // Implementation of BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    String[] tab = new String[inPullItfs.size() + 1];
    inPullItfs.keySet().toArray(tab);
    tab[inPullItfs.size()] = MessageManager.ITF_NAME;
    return tab;
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public synchronized void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.startsWith(IN_PULL_ITF_NAME))
    {
      inPullItfs.put(clientItfName, (Pull) serverItf);
    }
    else if (clientItfName.equals(MessageManager.ITF_NAME))
    {
      messageManagerItf = (MessageManager) serverItf;
    }
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#unbindFc(String)
   */
  public synchronized void unbindFc(String clientItfName)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.unbindFc(clientItfName);
    if (clientItfName.startsWith(IN_PULL_ITF_NAME))
    {
      inPullItfs.remove(clientItfName);
    }
  }
}
