/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.channel;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.PushException;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.MessageManager;
import org.objectweb.dream.protocol.ExportIdentifier;
import org.objectweb.dream.protocol.ExportIdentifierChunk;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * Destination resolver that retrieves {@link ExportIdentifier} if a
 * {@link org.objectweb.dream.protocol.ExportIdentifierChunk} contained in the
 * message.
 */
public class DestinationResolverChunkBasedImpl extends AbstractComponent
    implements
      DestinationResolver,
      DestinationResolverChunkBasedAttributeController
{

  // ---------------------------------------------------------------------------
  // Attribute fields
  // ---------------------------------------------------------------------------

  protected boolean        deleteChunk = true;
  protected String         destinationChunkName;

  // ---------------------------------------------------------------------------
  // Client interfaces
  // ---------------------------------------------------------------------------

  protected MessageManager messageManagerItf;

  // ---------------------------------------------------------------------------
  // Implementation of SocketManager interface
  // ---------------------------------------------------------------------------

  /**
   * @see DestinationResolver#resolveDestination(Message)
   */
  public ExportIdentifier resolveDestination(Message message)
      throws PushException
  {
    ExportIdentifierChunk chunk = (ExportIdentifierChunk) messageManagerItf.getChunk(message, destinationChunkName);
    if (chunk == null)
    {
      throw new PushException("Unable to find \"" + destinationChunkName
          + "\" chunk in message");
    }
    ExportIdentifier exportIdentifier = chunk.getExportIdentifier();
    if (deleteChunk)
    {
      messageManagerItf.removeChunk(message, destinationChunkName);
      messageManagerItf.deleteChunk(chunk);
    }
    return exportIdentifier;
  }

  // ---------------------------------------------------------------------------
  // Implementation of AttributeController interface
  // ---------------------------------------------------------------------------

  /**
   * @see DestinationResolverChunkBasedAttributeController#getDeleteChunk()
   */
  public boolean getDeleteChunk()
  {
    return deleteChunk;
  }

  /**
   * @see DestinationResolverChunkBasedAttributeController#setDeleteChunk(boolean)
   */
  public void setDeleteChunk(boolean deleteChunk)
  {
    this.deleteChunk = deleteChunk;
  }

  /**
   * @see DestinationResolverChunkBasedAttributeController#getDestinationChunkName()
   */
  public String getDestinationChunkName()
  {
    return destinationChunkName;
  }

  /**
   * @see DestinationResolverChunkBasedAttributeController#setDestinationChunkName(String)
   */
  public void setDestinationChunkName(String destinationChunkName)
  {
    this.destinationChunkName = destinationChunkName;
  }

  // ---------------------------------------------------------------------------
  // Implementation of BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{MessageManager.ITF_NAME};
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public synchronized void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(MessageManager.ITF_NAME))
    {
      messageManagerItf = (MessageManager) serverItf;
    }
  }
}
