/**
 * Dream Copyright (C) 2003-2004 INRIA Rhone-Alpes
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option) any
 * later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * Contact: dream@objectweb.org
 * 
 * Initial developer(s): Matthieu Leclercq 
 * Contributor(s):
 */

package org.objectweb.dream.control.activity;

import java.util.HashMap;
import java.util.Map;

import org.objectweb.dream.control.activity.task.IllegalTaskException;
import org.objectweb.dream.control.activity.task.Task;
import org.objectweb.dream.control.activity.task.TaskController;
import org.objectweb.dream.util.Dream;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;

/**
 * Utility methods to manage tasks.
 */
public final class Util
{

  private Util()
  {
  }

  /**
   * Adds the given task in the task controller of the given component.
   * 
   * @param component the component interface which should provide the
   *          {@link TaskController}control interface.
   * @param task the task to add
   * @param registrationHints the hints passed to the
   *          {@link TaskController#addTask(Task, Map)}method.
   * @throws NoSuchInterfaceException If the given component doesn't have a task
   *           controller interface.
   * @throws IllegalTaskException If the given task can't be added to the task
   *           controller.
   */
  public static void addTask(Component component, Task task,
      Map<String, Object > registrationHints) throws NoSuchInterfaceException,
      IllegalTaskException
  {
    TaskController taskController = Dream.getTaskController(component);
    if (registrationHints == null)
    {
      registrationHints = new HashMap<String, Object>();
    }
    taskController.addTask(task, registrationHints);
  }
}
