/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.control.activity.manager;

import java.util.HashMap;
import java.util.Map;

import org.objectweb.dream.control.activity.scheduler.PeriodicSchedulerImpl;
import org.objectweb.dream.control.activity.scheduler.Scheduler;
import org.objectweb.dream.control.activity.task.IllegalTaskException;
import org.objectweb.dream.control.activity.task.Task;
import org.objectweb.dream.util.Error;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.LifeCycleController;

/**
 * Scheduler manager for {@link PeriodicSchedulerImpl}.
 */
public class PeriodicSchedulerManagerImpl extends AbstractSchedulerManager
{
  static final String   PERIOD_HINTS_KEY  = "period";
  PeriodicSchedulerImpl periodicScheduler = null;
  Map<Task, String>     tasks             = new HashMap<Task, String>();
  long                  nextTaskId        = 0;

  // ---------------------------------------------------------------------------
  // Implementation of the SchedulerManager interface
  // ---------------------------------------------------------------------------

  /**
   * @see SchedulerManager#scheduleTask(Task, Map)
   */
  public Object scheduleTask(Task task, Map hints) throws IllegalTaskException
  {
    synchronized (tasks)
    {
      if (periodicScheduler == null)
      {
        try
        {
          periodicScheduler = new PeriodicSchedulerImpl();
          contentControllerItf.addFcSubComponent(periodicScheduler);
          Task thread = threadManagerItf.addThread();
          ((BindingController) periodicScheduler).bindFc(EndOfTask.ITF_NAME,
              contentControllerItf.getFcInternalInterface(EndOfTask.ITF_NAME));
          ((BindingController) thread).bindFc(Scheduler.ITF_NAME,
              periodicScheduler);
          ((BindingController) periodicScheduler).bindFc("logger", logger);
          ((LifeCycleController) thread).startFc();
        }
        catch (Exception e)
        {
          Error.bug(logger, e);
        }
      }
      Object o = hints.get(PERIOD_HINTS_KEY);
      if (o == null || !(o instanceof Number))
      {
        throw new IllegalTaskException(task, "Missing or invalid period");
      }
      String taskName = Task.ITF_NAME + (nextTaskId++);
      try
      {
        periodicScheduler.bindFc(taskName, task, o);
      }
      catch (Exception e)
      {
        Error.bug(logger, e);
      }
      tasks.put(task, taskName);
    }
    return null;
  }

  /**
   * @see SchedulerManager#unscheduleTask(Task)
   */
  public void unscheduleTask(Task task) throws IllegalTaskException
  {
    String taskName;
    synchronized (tasks)
    {
      taskName = tasks.remove(task);
    }
    if (taskName == null)
    {
      throw new IllegalTaskException(task, "unknown task");
    }
    try
    {
      periodicScheduler.unbindFc(taskName);
    }
    catch (Exception e)
    {
      Error.bug(logger, e);
    }
  }

  /**
   * @see org.objectweb.dream.control.activity.scheduler.InterruptTask#interruptTask(Task)
   */
  public void interruptTask(Task task) throws IllegalTaskException
  {
    if (periodicScheduler == null)
    {
      throw new IllegalTaskException(task, "unknown task");
    }
    periodicScheduler.interruptTask(task);
  }
}
