/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): 
 */

package org.objectweb.dream.control.location;

import org.objectweb.fractal.api.Component;

/**
 * A controller interface used to know the location of a component.
 */
public interface LocationController
{
  /**
   * Returns an array of the {@link Component }interface of bootstrap
   * components where the component is located. If the component this controller
   * belongs is a primitive component, this array contains one element : the
   * bootstrap used to create it. If the component is a composite component, the
   * returned array contains the bootstrap used to create it (at first element)
   * and every bootstraps of its sub components.
   * 
   * @return an array of the {@link Component }interface of bootstrap
   *         components where the component is located.
   */
  Component[] getBootstraps();

  /**
   * Method used by the bootstrap component during the initialisation of the
   * component.
   * 
   * @param bootstrap the bootstrap component interface that creates the
   *          component, this controller belongs.
   */
  void setCreationBootstrap(Component bootstrap);
}