/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.message;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;

import org.objectweb.dream.util.EmptyIterator;

class BasicMessageImpl extends Message
{
  private static final Iterator<String>           EMPTY_CHUNK_NAME_ITERATOR  = new EmptyIterator<String>();
  private static final Iterator<Message> EMPTY_SUB_MESSAGE_ITERATOR = new EmptyIterator<Message>();
  private final Map<String, AbstractChunk>        chunks;
  private LinkedList<Message>            subMessages;

  BasicMessageImpl(MessageManager owner)
  {
    super(owner);
    chunks = new HashMap<String, AbstractChunk>();
  }

  // ---------------------------------------------------------------------------
  // Implementation of the Recyclable interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.pool.Recyclable#recycle()
   */
  public void recycle()
  {
    if (!chunks.isEmpty())
    {
      chunks.clear();
    }
    if (subMessages != null && !subMessages.isEmpty())
    {
      subMessages.clear();
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of the MessageReference abstract methods
  // ---------------------------------------------------------------------------

  @Override
  protected boolean isEmpty()
  {
    return chunks.isEmpty() && (subMessages == null || subMessages.isEmpty());
  }

  @Override
  protected AbstractChunk getChunk(final String name)
  {
    return chunks.get(name);
  }

  @Override
  protected void addChunk(final String name, final AbstractChunk chunk)
  {
    chunks.put(name, chunk);
  }

  @Override
  protected void removeChunk(final String name)
  {
    chunks.remove(name);
  }

  @Override
  protected Iterator<String> getNameIterator()
  {
    if (chunks.size() == 0)
    {
      return EMPTY_CHUNK_NAME_ITERATOR;
    }
    return chunks.keySet().iterator();
  }

  @Override
  protected void addSubMessage(final Message subMessage)
  {
    if (subMessages == null)
    {
      subMessages = new LinkedList<Message>();
    }
    subMessages.addLast(subMessage);
  }

  @Override
  protected Iterator<Message> getSubMessageIterator()
  {
    if (subMessages == null || subMessages.size() == 0)
    {
      return EMPTY_SUB_MESSAGE_ITERATOR;
    }
    return subMessages.iterator();
  }

  @Override
  protected boolean isSubMessage(final Message subMessage)
  {
    if (subMessages == null)
    {
      return false;
    }
    return subMessages.contains(subMessage);
  }

  @Override
  protected boolean removeSubMessage(final Message subMessage)
  {
    if (subMessages == null)
    {
      return false;
    }
    return subMessages.remove(subMessage);
  }
}
