/**
 * DREAM
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s):
 */

package org.objectweb.dream.message.codec;

import java.io.IOException;
import java.lang.ref.Reference;

import org.objectweb.dream.pool.Recyclable;

/**
 * Basic implementation of the {@link CodecInputOutput} interface.
 */
public class CodecInputOutputImpl implements CodecInputOutput, Recyclable
{

  protected Object                         input;
  protected Object                         output;
  private Reference< ? extends Recyclable> recyclableReference;

  /** Default constructor */
  public CodecInputOutputImpl()
  {
  }

  /**
   * @param input the input
   * @param output the output
   */
  public CodecInputOutputImpl(Object input, Object output)
  {
    this.input = input;
    this.output = output;
  }

  // ---------------------------------------------------------------------------
  // Implementation of the CondecInputOutput interface
  // ---------------------------------------------------------------------------

  /**
   * @see CodecInputOutput#getInput()
   */
  public Object getInput() throws IOException
  {
    return input;
  }

  /**
   * @see CodecInputOutput#setInput(Object)
   */
  public void setInput(Object input)
  {
    this.input = input;
  }

  /**
   * @see CodecInputOutput#getOutput()
   */
  public Object getOutput() throws IOException
  {
    return output;
  }

  /**
   * @see CodecInputOutput#setOutput(Object)
   */
  public void setOutput(Object output)
  {
    this.output = output;
  }

  // ---------------------------------------------------------------------------
  // Implementation of the Recyclable interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.pool.Recyclable#recycle()
   */
  public void recycle()
  {
    input = null;
    output = null;
  }

  /**
   * @see Recyclable#setReference(Reference)
   */
  public void setReference(Reference< ? extends Recyclable> reference)
  {
    this.recyclableReference = reference;
  }

  /**
   * @see Recyclable#getReference()
   */
  public Reference< ? extends Recyclable> getReference()
  {
    return recyclableReference;
  }
}
