/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.message.codec;

import java.io.IOException;

import org.objectweb.dream.message.Message;

/**
 * Message management interface to encode/decode messages on
 * {@link CodecInputOutput}.
 */
public interface MessageCodec
{

  /** The commonly used name of refer to this interface. */
  String ITF_NAME = "message-codec";

  /**
   * Encodes the given message on the given stream. This method is responsible
   * to encode the given message and its sub messages recursively.
   * 
   * @param cio contains the input on which the message is encoded.
   * @param message a message to serialize.
   * @throws IOException if an error occurs during the serialization.
   */
  void encode(CodecInputOutput cio, Message message)
      throws IOException;

  /**
   * De-serializes a message from the given stream. This method is responsible
   * to decode the given message and its sub messages recursively.
   * 
   * @param cio contains the output on which the message is decoded.
   * @return a message
   * @throws IOException if an error occurs during the de-serialization.
   */
  Message decode(CodecInputOutput cio) throws IOException;
}
