/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.protocol;

import java.io.IOException;

import org.objectweb.dream.IOPushException;
import org.objectweb.dream.message.Message;

/**
 * Push interface on which <i>outgoing messages</i> are passed to a protocol.
 * Outgoing messages are messages going down the protocol stack.
 * 
 * @see IncomingPush
 * @see org.objectweb.dream.Push
 */
public interface OutgoingPush
{

  /**
   * Gives an outgoing message to a protocol.
   * 
   * @param message the message to give to the component.
   * @throws IOPushException if an exception occurs.
   */
  void outgoingPush(Message message) throws IOPushException;

  /**
   * Closes the communication channel. The implementation of this method must
   * not call the {@link IncomingPush#incomingClosed} method, since this latter
   * is only used to inform a upper session that a lower session has been closed
   * unexpectedly.
   * 
   * @param incomingPush the interface on which incoming messages of the closed
   *          communication channel were received.
   * @throws IOException the closing process failed.
   */
  void outgoingClose(IncomingPush incomingPush) throws IOException;
}
