/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.protocol.bus;

import org.objectweb.dream.protocol.BindException;
import org.objectweb.dream.protocol.ExportException;
import org.objectweb.dream.protocol.ExportIdentifier;
import org.objectweb.dream.protocol.IncomingPush;
import org.objectweb.dream.protocol.messagePassing.MessagePassingOutgoingPush;

/**
 * Interface used by bus protocol to try to determine the local address (if not
 * specified in binding hints see {@link BusProtocol#LOCAL_ADDRESS_HINT_KEY}),
 * and to export the access point of the local bus session.
 */
public interface LocalAccessManager
{
  /** Default name for interfaces of this type. */
  String ITF_NAME = "local-access-manager";

  /**
   * Tries to find in the given array of export identifier, the one which
   * correspond to the local protocol.
   * 
   * @param exportIds an array of export identifier.
   * @return the index in the given array of the export identifier corresponding
   *         to the local protocol.
   * @throws BindException if no export identifier can be found or if two, or
   *           more export identifiers may correspond to the local protocol.
   */
  int findLocalAddress(ExportIdentifier[] exportIds) throws BindException;

  /**
   * Export to the lower level protocol in order to create the access point of
   * the local bus session. The local export identifier of the created access
   * point (see
   * {@link org.objectweb.dream.protocol.messagePassing.MessagePassingOutgoingPush#getLocalExportIdentifier()})
   * must be equivalent to the given export identifier.
   * 
   * @param localId the identifier to obtain for the given channel.
   * @param incomingPush the interface on which bus session want to receive
   *          incoming messages.
   * @return the access point of the local bus session.
   * @throws ExportException if the export process fails.
   */
  MessagePassingOutgoingPush exportBus(ExportIdentifier localId,
      IncomingPush incomingPush) throws ExportException;
}
