/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.protocol.channel;

import org.objectweb.fractal.api.control.AttributeController;

/**
 * Attribute controller for TCP/IP protocol. Defines default values for export
 * and bind hints.
 * 
 * @see TCPIPProtocol
 */
public interface TCPIPProtocolAttributeController extends AttributeController
{
  /** Default value of <code>connectionRetry</code> attribute. */
  int CONNECTION_RETRY_DEFAULT = 5;

  /**
   * Returns the default value of the {@link TCPIPProtocol#PORT} hint.
   * 
   * @return the <code>port</code> attribute's value.
   * @see TCPIPProtocol#PORT
   * @see #setPort(int)
   */
  int getPort();

  /**
   * Sets the default value of the {@link TCPIPProtocol#PORT} hint.
   * 
   * @param port the new <code>port</code> attribute's value.
   * @see TCPIPProtocol#PORT
   * @see #getPort()
   */
  void setPort(int port);

  /**
   * Returns the default value of the {@link TCPIPProtocol#CONNECTION_RETRY}
   * hint.
   * 
   * @return {@link #CONNECTION_RETRY_DEFAULT} by default, or the value set by
   *         {@link #setConnectionRetry}.
   * @see TCPIPProtocol#CONNECTION_RETRY
   * @see #setConnectionRetry(int)
   */
  int getConnectionRetry();

  /**
   * Sets the default value of the {@link TCPIPProtocol#CONNECTION_RETRY} hint.
   * 
   * @param connectionRetry the new <code>connectionRetry</code> attribute's
   *          value.
   * @see TCPIPProtocol#CONNECTION_RETRY
   * @see #getConnectionRetry()
   */
  void setConnectionRetry(int connectionRetry);

  /**
   * Returns the default value of the {@link TCPIPProtocol#TCP_NO_DELAY} hint.
   * 
   * @return the <code>tcpNoDelay</code> attribute's value.
   * @see TCPIPProtocol#TCP_NO_DELAY
   * @see java.net.Socket#setTcpNoDelay(boolean)
   * @see #setTcpNoDelay(boolean)
   */
  boolean getTcpNoDelay();

  /**
   * Sets the default value of the {@link TCPIPProtocol#TCP_NO_DELAY} hint.
   * 
   * @param tcpNoDelay the new <code>tcpNoDelay</code> attribute's value.
   * @see TCPIPProtocol#TCP_NO_DELAY
   * @see #getTcpNoDelay()
   */
  void setTcpNoDelay(boolean tcpNoDelay);

  /**
   * Returns the default value of the {@link TCPIPProtocol#SO_TIMEOUT} hint.
   * 
   * @return the <code>soTimeout</code> attribute's value.
   * @see TCPIPProtocol#SO_TIMEOUT
   * @see java.net.Socket#setSoTimeout(int)
   * @see #setSoTimeout(int)
   */
  int getSoTimeout();

  /**
   * Sets the default value of the {@link TCPIPProtocol#SO_TIMEOUT} hint.
   * 
   * @param timeout the new <code>soTimeout</code> attribute's value.
   * @see TCPIPProtocol#SO_TIMEOUT
   * @see java.net.Socket#setSoTimeout(int)
   * @see #getSoTimeout()
   */
  void setSoTimeout(int timeout);

  /**
   * Returns the default value of the {@link TCPIPProtocol#SO_LINGER} hint.
   * 
   * @return the <code>soLinger</code> attribute's value.
   * @see TCPIPProtocol#SO_LINGER
   * @see java.net.Socket#setSoLinger(boolean, int)
   * @see #setSoLinger(int)
   */
  int getSoLinger();

  /**
   * Sets the default value of the {@link TCPIPProtocol#SO_LINGER} hint.
   * 
   * @param timeout the new <code>soLinger</code> attribute's value.
   * @see TCPIPProtocol#SO_LINGER
   * @see java.net.Socket#setSoLinger(boolean, int)
   * @see #getSoLinger()
   */
  void setSoLinger(int timeout);

}