/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.protocol.messagePassing;

import org.objectweb.fractal.api.control.AttributeController;

/**
 * Attribute controller interface of UDP protocol implementation
 * 
 * @see UDPProtocol
 * @see UDPProtocolImpl
 */
public interface UDPProtocolAttributeController extends AttributeController
{
  /**
   * Returns the default value of the {@link UDPProtocol#PORT} hint.
   * 
   * @return the <code>port</code> attribute's value.
   * @see UDPProtocol#PORT
   * @see #setPort(int)
   */
  int getPort();

  /**
   * Sets the default value of the {@link UDPProtocol#PORT} hint.
   * 
   * @param port the new <code>port</code> attribute's value.
   * @see UDPProtocol#PORT
   * @see #getPort()
   */
  void setPort(int port);

  /**
   * Sets the size of buffer used in {@link java.net.DatagramPacket} when
   * receiving message.
   * 
   * @param recvBufferSize the new <code>recvBufferSize</code> attribute
   *          value.
   * @see java.net.DatagramPacket#DatagramPacket(byte[], int)
   */
  void setRecvBufferSize(int recvBufferSize);

  /**
   * Returns the <code>recvBufferSize</code> attribute value.
   * 
   * @return the <code>recvBufferSize</code> attribute value.
   * @see #setRecvBufferSize(int)
   */
  int getRecvBufferSize();
}
