/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): Romain Lenglet
 */

package org.objectweb.dream.pushwithreturn;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.Push;
import org.objectweb.dream.PushException;
import org.objectweb.dream.message.MessageManager;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * Basic implementation of the {@link HandleReturnMessage}interface. The
 * component simply forwards return messages.
 */
public class HandleReturnMessageImpl extends AbstractComponent
    implements
      HandleReturnMessage
{

  // Client interfaces
  /** The client interface used to access the message manager. */
  MessageManager messageManagerItf;

  /** The Push output interface. */
  Push           outPushItf;

  // ---------------------------------------------------------------------------
  // Implementation of the HandleReturnMessage interface.
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.pushwithreturn.HandleReturnMessage#handleReturnMessage(org.objectweb.dream.pushwithreturn.WaitingKey)
   */
  public void handleReturnMessage(WaitingKey waitingKey) throws Exception
  {

    try
    {
      outPushItf.push(waitingKey.getReturnMessage());
      messageManagerItf.deleteMessage(waitingKey.getMessage());
    }
    catch (PushException e)
    {
      throw new Exception(e);
    }

  }

  // ---------------------------------------------------------------------------
  // Implementation of the BindingController interface.
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(Push.OUT_PUSH_ITF_NAME))
    {
      outPushItf = (Push) serverItf;
    }
    else if (clientItfName.equals(MessageManager.ITF_NAME))
    {
      messageManagerItf = (MessageManager) serverItf;
    }
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{Push.OUT_PUSH_ITF_NAME, MessageManager.ITF_NAME};
  }

}