/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.queue;

import org.objectweb.fractal.api.control.AttributeController;

/**
 * Base attribute controller interface for buffer implementations. It defines a
 * read only attribute which is the current size of the buffer. It also defines
 * the maximum capacity of the buffer. A maximum capacity equals to zero
 * indicates that there is no capacity limitation.
 */
public interface BufferAttributeController extends AttributeController
{

  /**
   * Returns the current size of the queue.
   * 
   * @return the current size of the queue.
   */
  int getCurrentSize();

  /**
   * Returns the maximum capacity of the queue.
   * 
   * @return the maximum capacity of the queue.
   */
  int getMaxCapacity();

  /**
   * Sets the maximum capacity of the queue.
   * 
   * @param maxCapacity the maximum capacity of the queue.
   * @throws IllegalArgumentException if <code>maxCapacity</code> is negative.
   * @throws InterruptedException if it is interrupted while waiting for the
   *           mutex to be acquired.
   */
  void setMaxCapacity(int maxCapacity) throws InterruptedException;

}