/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): 
 */

package org.objectweb.dream.util;

import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * Defines some methods to handle (i.e. throw exception) errors.
 */
public final class Error
{

  private Error()
  {
  }

  /**
   * Handles a bug: throws an {@link InternalError}and logs a
   * <code>ERROR</code> message.
   * 
   * @param logger the logger to be used to log the error message.
   */
  public static void bug(Logger logger)
  {
    if (logger != null)
    {
      logger.log(BasicLevel.ERROR,
          "This is a bug. Please contact dream@objectweb.org");
    }
    throw new InternalError("This is a bug. Please contact dream@objectweb.org");
  }

  /**
   * Handles a bug: throws an {@link InternalError}and logs a
   * <code>ERROR</code> message.
   * 
   * @param logger the logger to be used to log the error message.
   * @param e a throwable object.
   */
  public static void bug(Logger logger, Throwable e)
  {
    if (logger != null && logger.isLoggable(BasicLevel.ERROR))
    {
      logger.log(BasicLevel.ERROR,
          "This is a bug. Please contact dream@objectweb.org", e);
    }
    else
    {
      e.printStackTrace();
    }
    throw new InternalError("This is a bug. Please contact dream@objectweb.org");
  }

  /**
   * Handles an error: throws an {@link InternalError}and logs a
   * <code>ERROR</code> message.
   * 
   * @param message the message to display.
   * @param logger the logger to be used to log the error message.
   * @param e a throwable object.
   */
  public static void error(String message, Logger logger, Throwable e)
  {
    if (logger != null && logger.isLoggable(BasicLevel.ERROR))
    {
      logger.log(BasicLevel.ERROR, message, e);
    }
    else
    {
      e.printStackTrace();
    }
    throw new InternalError(message);
  }

  /**
   * Handles an error: throws an {@link InternalError}and logs a
   * <code>ERROR</code> message.
   * 
   * @param message the message to display.
   * @param logger the logger to be used to log the error message.
   */
  public static void error(String message, Logger logger)
  {
    if (logger != null)
    {
      logger.log(BasicLevel.ERROR, message);
    }
    throw new InternalError(message);
  }

}